<%Header
  title="XAO::Web Default Template"
  keywords="xao e-commerce opensource perl dynamic template web service apache"
  description="XAO::Web Default Template"
%>
<BODY>

<STRONG>This is the default template for XAO::Web -- XAO Web Developer.
</STRONG>
<P>

If you see this page instead of a site then the site is probably misconfigured
-- XAO::Web engine works, but templates are not available.
<P>

The time now is <EM><%Date/h style="short"%></EM>.
<BR>
The current URL is <EM><%Utility/h mode="current-url"%></EM>.
<P>

Here is the source of this page in case you're interested:
<TABLE BORDER BGCOLOR="#DDDDDD"><TR><TD><PRE>
<%Page/html path="/index.html" unparsed%>
</PRE></TD></TR></TABLE>

<%Footer%>
